function [ x, T ] = simplex( A, b, c, eq)
%simplex method to solve LP problem:
% max  c'x
% s.t. Ax {<=,=,>=} b; b>=0; x>=0
%
%inputs:
%   A [mxn] ... constraint coefficients
%   b [mx1] ... constraint bounds
%   c [nx1] ... objective coefficients
%   eq[mx1] ... (in)equality flag
%               +1 -> less equal (<=)
%                0 -> equal (=)
%               -1 -> greater equal (>=)
%
%output:
%   x [nx1]         ... optimal solution vector
%   T [m+n+1 x m+1] ... final tableau

  n = size(A,2);
  m = size(A,1);
  
  %vectorize
  if size(c,1) == 1
      c = c(:);
  end
  if size(b,1) == 1
      b = b(:);
  end
  
  %correct non-feasible constraints (b<0)
  nf = find(b<0);
  A(nf,:) = A(nf,:)*(-1);
  b(nf) = b(nf)*(-1);
  eq(nf) = eq(nf)*(-1);
  
  %indices of artificial vars
  artificial = find(eq<=0);
  
  %if there are artificial vars, use two-phase-method
  if ~isempty(artificial)
    %additional columns for surplus vars
    surplus = zeros(m,sum(eq<0)); i = 1;
    for j = find(eq<0)
        surplus(j,i) = -1;
        i = i + 1;
    end
  
    %build tableau for phase I
    I = [eye(m),surplus];  
    r = zeros(1,n+m+size(surplus,2)+1); r(artificial+n) = 1;
    tableau_phase_I = [A, I, b; r]; 
   
    %build tableau for phase II
    tableau_phase_II = simplexiter(tableau_phase_I);
    tableau_phase_II(:,artificial+n) = [];
    tableau_phase_II(m+1,:) = [ -c', zeros(1,sum(eq~=0)+1) ];
  else
    %there are no artificial vars, so just do phase II
    tableau_phase_II = [A,eye(m),b; -c', zeros(1,m+1)];
  end

  T = simplexiter(tableau_phase_II);
  %extract solution vector from final tableau
  x = zeros(n,1);
  for i=1:n
     if sum(T(1:m,i)==1)==1 && sum(T(1:m,i)==0)==m-1 
         x(i) = T(T(:,i)==1,m+n+1);
     end
  end
  
end